<?php

namespace Drupal\commerce_icici_pay\PluginForm\IciciRedirect;

use Drupal\commerce_icici_pay\IciciUtility;
use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm as BasePaymentOffsiteForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\commerce_order\Entity\Order;

class IciciPaymentForm extends BasePaymentOffsiteForm {

    /**
     * {@inheritdoc}
     */
    public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
        $form = parent::buildConfigurationForm($form, $form_state);

        /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
        $payment = $this->entity;

        /** @var \Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayInterface $payment_gateway_plugin */
        $payment_gateway_plugin = $payment->getPaymentGateway()->getPlugin();

        $order_id = \Drupal::routeMatch()->getParameter('commerce_order')->id();
        $order = Order::load($order_id);
        $address = $order->getBillingProfile()->address->first();
        $profile = $order->getBillingProfile();
        $mode = $payment_gateway_plugin->getConfiguration()['mode'];
        
        // Get key and merchant_id of bank according to center id
        $merchant_id = $payment_gateway_plugin->getConfiguration()['merchant_id'];
        $working_key = $payment_gateway_plugin->getConfiguration()['secret_key'];

        $voucher_id = \Drupal::config('aakash_common.settings')->get('voucher_id').$order_id;
        $return_url = Url::FromRoute('commerce_payment.checkout.return', ['commerce_order' => $order_id, 'step' => 'payment'], array('absolute' => TRUE))->toString();
        $parameters = [
            'merchantTxnNo' => $order_id.time(),
            'invoiceNo' => $voucher_id,
            'amount' => round($payment->getAmount()->getNumber(), 2),
            'txnDate' => date('YmdHis'),
            'returnURL' => $return_url,
            'customerName' => $profile->get('field_product_student_fname')->value,
            'customerEmailID' => $order->getEmail(),
            'customerMobileNo' => $profile->get('field_phone')->value,
            // 'addline1' => $address->getAddressLine1(),
            // 'cityname' => $address->getLocality(),
            // 'state' => $address->getAdministrativeArea(),
            // 'zipCode' => $address->getPostalCode(),
            // 'country' => $address->getCountryCode() ?? 'IN',
        ];
        $parameters['merchantId'] = $merchant_id;
        $parameters['payType'] = ICICI_PAYMENT_ARR['p_sale_pay_type'];
        $parameters['transactionType'] = ICICI_PAYMENT_ARR['p_sale_txn_type'];
        $parameters['currencyCode'] = ICICI_PAYMENT_ARR['p_currency_code'];

        // To generate secure hash using hmac method
        ksort($parameters);
        $hashString = '';
        foreach ($parameters as $key => $value) {
            $hashString .= $value; 
        }
        $utility = new IciciUtility();
        $secureHash = $utility->hmacDigest($hashString, $working_key);
        $parameters['secureHash'] = $secureHash;

        // API to initiate payment and redirect to payment gateway
        if ($mode == 'test') {
            $api_url = ICICI_SALE_API_URL_pay;
        }
        else {
            $api_url = ICICI_SALE_API_URL;
        }
        $api_res = $utility->curlPostApi($api_url, $parameters);
        $redirect_method = 'get';
        $redirect_url = '';
        $params = [];
        // Check for API response
        if ($api_res && $api_res->responseCode == 'R1000') {
            $redirect_url = $api_res->redirectURI;
            $params['tranCtx'] = $api_res->tranCtx;
        }
        else {
            drupal_set_message(t('Something went wrong. Please try again.'), 'error');
        }

        // Store logs
        store_log_data(serialize($parameters), 'icici', 'payment-parameters', $order_id);

        return $this->buildRedirectForm($form, $form_state, $redirect_url, $params, $redirect_method);
    }

}